#include <acknex.h>

// Material skills
#define skPosX       skill17 // X coord in map coords relative to the center of the map (centered on the screen)
#define skPosY       skill18 // Y coord...
#define skScale      skill19 // scale of the map
#define skScaleNext  skill20 // needed a map scale smooth transition variable

FONT *fntArial = "Arial#64";
var nTest = 0;
PANEL *panObj = 
{
	digits ( 15, 0, "%.0f", fntArial, 1, nTest );
	blue = 0;
	green = 0;
	red = 0;
}

TEXT *txtObj = 
{ 
	string = "0";
	font = fntArial;
	blue = 0;
	green = 255;
	red = 0;
}

#define CHECKER_SIZE 128
void draw_map ( BMAP *bmpMap )
{
	bmap_rendertarget ( bmpMap, 0, 0 );
		draw_quad ( NULL, vector(0,0,0), NULL, vector(bmap_width(bmpMap),bmap_height(bmpMap),0), NULL, COLOR_WHITE, 100, 0 );
		int posX = 0;
		for ( ; posX<bmap_width(bmpMap); posX+=CHECKER_SIZE )
		{
			int posY = 0;
			for ( ; posY<bmap_height(bmpMap); posY+=CHECKER_SIZE )
			{
				if ( ( ( posX + posY ) / CHECKER_SIZE ) % 2 )
				{
					draw_quad ( NULL, vector(posX,posY,0), NULL, vector(CHECKER_SIZE,CHECKER_SIZE,0), NULL, COLOR_BLUE, 100, 0 );
					txtObj->pos_x = posX + 15 + CHECKER_SIZE * 0.5;
					txtObj->pos_y = posY;
					draw_obj ( txtObj );
				}
				else
				{
					draw_text ( "0", posX+15, posY+CHECKER_SIZE*0.5, COLOR_RED );
					panObj->pos_x = posX + CHECKER_SIZE * 0.5;
					panObj->pos_y = posY;
					draw_obj ( panObj );
				}
				
				draw_line ( vector(posX,posY,0), COLOR_BLACK, 0 );
				draw_line ( vector(posX,posY,0), COLOR_BLACK, 100 );
				draw_line ( vector(posX+CHECKER_SIZE,posY+CHECKER_SIZE,0), COLOR_BLACK, 100 );
				draw_line ( vector(posX+CHECKER_SIZE,posY+CHECKER_SIZE,0), COLOR_BLACK, 0 );
			}
		}
	bmap_rendertarget ( NULL, 0, 0 );
}

void main ()
{
	video_mode = 8;
	mouse_mode = 4;
	fps_max = 60;
	wait(1);
	
	draw_textmode ( "Arial", 0, 64, 100 );
	bmap_zbuffer ( bmap_createblack ( 2048, 2048, 32 ) );
	wait(1);
	
	BMAP *bmpMap = bmap_createblack ( 1900, 1200, 32 );
	draw_map ( bmpMap );
	
	BMAP *bmpScreen = bmap_createblack ( screen_size.x, screen_size.y, 24 );
	PANEL *panScreen = pan_create ( "flags=SHOW", 1 );
	panScreen->bmap = bmpScreen;
	
	MATERIAL *mtlCamMap = mtl_create ();
	mtlCamMap->skPosX = 0;
	mtlCamMap->skPosY = 0;
	mtlCamMap->skScale = 0.25;
	mtlCamMap->skScaleNext = mtlCamMap->skScale;
	mtlCamMap->skill1 = floatv ( bmap_width(bmpScreen) );
	mtlCamMap->skill2 = floatv ( bmap_height(bmpScreen) );
	mtlCamMap->skill3 = floatd ( 1, bmap_width(bmpMap) );
	mtlCamMap->skill4 = floatd ( 1, bmap_height(bmpMap) );
	effect_load ( mtlCamMap, "display_map.fx" );
	
	while ( !key_esc )
	{
		if ( mouse_left )
		{
			mtlCamMap->skPosX -= mickey.x / mtlCamMap->skScale;
			mtlCamMap->skPosY -= mickey.y / mtlCamMap->skScale;
		}
		if ( mickey.z > 0 )
		{
			mtlCamMap->skScaleNext = minv ( mtlCamMap->skScaleNext * 1.414, 2 ); // 1.414 ~= sqrt(2)
		}
		if ( mickey.z < 0 )
		{
			mtlCamMap->skScaleNext = maxv ( mtlCamMap->skScaleNext / 1.414, 0.25 );
		}
		var nTemp = ( mtlCamMap->skScaleNext - mtlCamMap->skScale ) * time_step * 0.8;
		if ( nTemp )
			mtlCamMap->skScale += nTemp;
		else
			mtlCamMap->skScale = mtlCamMap->skScaleNext;
		
		mtlCamMap->skill5 = floatv ( mtlCamMap->skPosX );
		mtlCamMap->skill6 = floatv ( mtlCamMap->skPosY );
		mtlCamMap->skill7 = floatd ( 1, mtlCamMap->skScale );
		bmap_process ( bmpScreen, bmpMap, mtlCamMap );
		
		wait(1);
	}
	
	mtl_remove ( mtlCamMap );
	pan_remove ( panScreen );
	bmap_remove ( bmpScreen );
	bmap_remove ( bmpMap );
	
	sys_exit ( NULL );
}